# -*- coding: utf-8 -*-

import os
import pathlib
import subprocess

from lintwork.work.abstract import WorkAbstract
from lintwork.proto.proto import Format

LINT_LEN_MIN = 3
LINT_SEP = ":"


class CpplintException(Exception):
    def __init__(self, info):
        super().__init__(self)
        self._info = info

    def __str__(self):
        return self._info


class Cpplint(WorkAbstract):
    def __init__(self, config):
        if config is None:
            config = []
        super().__init__(config)

    def _execution(self, project):
        return self._lint(project)

    def _parse(self, data):
        buf = []
        for item in data.splitlines():
            b = item.strip().split(LINT_SEP)
            if len(b) < LINT_LEN_MIN:
                continue
            buf.append(
                {
                    Format.FILE: b[0].strip(),
                    Format.LINE: int(b[1].strip()),
                    Format.TYPE: "",
                    Format.DETAILS: " ".join(b[2:]).strip(),
                }
            )
        return buf

    def _popen(self, cmd, stdin=None):
        return subprocess.Popen(
            cmd, stdin=stdin, stdout=subprocess.PIPE, stderr=subprocess.PIPE
        )

    def _lint(self, project):
        def _helper(name):
            cmd = ["cpplint"]
            cmd.extend(self._config)
            cmd.extend([name])
            with self._popen(cmd) as proc:
                out, err = proc.communicate()
                if proc.returncode == 0:
                    return []
            return self._parse(
                err.strip().decode("utf-8").replace(project + os.path.sep, "")
            )

        buf = []
        for item in pathlib.Path(project).glob("**/*"):
            if item.is_file():
                b = _helper(item)
                if len(b) != 0:
                    buf.extend(b)
        return buf



# -*- coding: utf-8 -*-

import sys

from lintwork.cmd.argument import Argument
from lintwork.cmd.banner import BANNER
from lintwork.config.config import Config, ConfigException
from lintwork.lint.lint import Lint, LintException
from lintwork.logger.logger import Logger
from lintwork.queue.queue import Queue, QueueException
from lintwork.work.work import Work, WorkException


def main():
    print(BANNER)

    argument = Argument()
    arg = argument.parse(sys.argv)

    try:
        config = Config()
        config.config_file = arg.config_file
        config.lint_project = arg.lint_project
        config.listen_url = arg.listen_url
        config.output_file = arg.output_file
    except ConfigException as e:
        Logger.error(str(e))
        return -1

    try:
        work = Work(config)
    except WorkException as e:
        Logger.error(str(e))
        return -2

    Logger.info("lint running")

    if len(config.listen_url) != 0:
        try:
            lint = Lint(config)
            lint.run(work.routine)
        except LintException as e:
            Logger.error(str(e))
            return -3
    else:
        try:
            queue = Queue(config)
            queue.run(work.routine, config.lint_project)
        except QueueException as e:
            Logger.error(str(e))
            return -4

    Logger.info("lint exiting")

    return 0



# -*- coding: utf-8 -*-

import base64
import datetime
import grpc
import json
import os
import pathlib
import shutil

from concurrent import futures
from lintwork.lint.lint_pb2 import LintReply
from lintwork.lint.lint_pb2_grpc import (
    add_LintProtoServicer_to_server,
    LintProtoServicer,
)

LINT_NAME = "lintwork"
MAX_WORKERS = 10


class LintException(Exception):
    def __init__(self, info):
        super().__init__(self)
        self._info = info

    def __str__(self):
        return self._info


class Lint(object):
    def __init__(self, config):
        if config is None:
            raise LintException("config invalid")
        self._config = config

    def _serve(self, routine):
        server = grpc.server(futures.ThreadPoolExecutor(max_workers=MAX_WORKERS))
        add_LintProtoServicer_to_server(LintProto(routine), server)
        server.add_insecure_port(self._config.listen_url)
        server.start()
        server.wait_for_termination()

    def run(self, routine):
        self._serve(routine)


class LintProto(LintProtoServicer):
    def __init__(self, routine):
        self._routine = routine

    def _build(self, data):
        def _helper(root, dirs, file, data):
            pathlib.Path(os.path.join(root, dirs)).mkdir(parents=True, exist_ok=True)
            p = pathlib.Path(os.path.join(root, dirs, file))
            with p.open("w", encoding="utf-8") as f:
                f.write(base64.b64decode(data).decode("utf-8"))

        if len(data) == 0:
            return None
        buf = json.loads(data)
        root = os.path.join(
            os.getcwd(),
            LINT_NAME + "-" + datetime.datetime.now().strftime("%Y%m%d%H%M%S"),
        )
        pathlib.Path(root).mkdir(parents=True, exist_ok=True)
        for key, val in buf.items():
            _helper(
                root,
                os.path.dirname(key),
                os.path.basename(key).replace(".base64", ""),
                val,
            )

        return root

    def _clean(self, project):
        if os.path.exists(project):
            shutil.rmtree(project)

    def SendLint(self, request, _):
        if len(request.message) == 0:
            return LintReply(message="")
        project = self._build(request.message.strip())
        if project is None or not os.path.exists(project):
            return LintReply(message="")
        buf = self._routine(project)
        self._clean(project)
        return LintReply(message=json.dumps({LINT_NAME: buf}))



# -*- coding: utf-8 -*-

import os

from lintwork.config.config import ConfigFile
from lintwork.printer.printer import Printer
from lintwork.work.commit.contentcheck import Contentcheck  # noqa: F401
from lintwork.work.commit.messagecheck import Messagecheck  # noqa: F401
from lintwork.work.cpp.checkpatch import Checkpatch  # noqa: F401
from lintwork.work.cpp.cpplint import Cpplint  # noqa: F401
from lintwork.work.java.aosp import Aosp  # noqa: F401
from lintwork.work.java.checkstyle import Checkstyle  # noqa: F401
from lintwork.work.python.flake8 import Flake8  # noqa: F401
from lintwork.work.shell.shellcheck import Shellcheck  # noqa: F401


class WorkException(Exception):
    def __init__(self, info):
        super().__init__(self)
        self._info = info

    def __str__(self):
        return self._info


class Work(object):
    def __init__(self, config):
        if config is None:
            raise WorkException("config invalid")
        self._config = config
        self._spec = config.config_file.get(ConfigFile.SPEC, None)
        if self._spec is None:
            raise WorkException("spec invalid")

    def _dump(self, data):
        printer = Printer()
        printer.run(data=data, name=self._config.output_file, append=False)

    def routine(self, project):
        if not isinstance(project, str) or not os.path.exists(project):
            raise WorkException("project invalid")
        buf = []
        for key in self._spec.keys():
            for k, v in self._spec[key].items():
                cls = globals().get(k.capitalize(), None)
                if cls is not None:
                    buf.extend(cls(v).run(project))
        if len(self._config.output_file) != 0:
            self._dump(buf)
        return buf



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: lint.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor.FileDescriptor(
    name="lint.proto",
    package="lint",
    syntax="proto3",
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n\nlint.proto\x12\x04lint"\x1e\n\x0bLintRequest\x12\x0f\n\x07message\x18\x01 \x01(\t"\x1c\n\tLintReply\x12\x0f\n\x07message\x18\x01 \x01(\t2=\n\tLintProto\x12\x30\n\x08SendLint\x12\x11.lint.LintRequest\x1a\x0f.lint.LintReply"\x00\x62\x06proto3',
)


_LINTREQUEST = _descriptor.Descriptor(
    name="LintRequest",
    full_name="lint.LintRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="message",
            full_name="lint.LintRequest.message",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=20,
    serialized_end=50,
)


_LINTREPLY = _descriptor.Descriptor(
    name="LintReply",
    full_name="lint.LintReply",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="message",
            full_name="lint.LintReply.message",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=52,
    serialized_end=80,
)

DESCRIPTOR.message_types_by_name["LintRequest"] = _LINTREQUEST
DESCRIPTOR.message_types_by_name["LintReply"] = _LINTREPLY
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

LintRequest = _reflection.GeneratedProtocolMessageType(
    "LintRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _LINTREQUEST,
        "__module__": "lint_pb2"
        # @@protoc_insertion_point(class_scope:lint.LintRequest)
    },
)
_sym_db.RegisterMessage(LintRequest)

LintReply = _reflection.GeneratedProtocolMessageType(
    "LintReply",
    (_message.Message,),
    {
        "DESCRIPTOR": _LINTREPLY,
        "__module__": "lint_pb2"
        # @@protoc_insertion_point(class_scope:lint.LintReply)
    },
)
_sym_db.RegisterMessage(LintReply)


_LINTPROTO = _descriptor.ServiceDescriptor(
    name="LintProto",
    full_name="lint.LintProto",
    file=DESCRIPTOR,
    index=0,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_start=82,
    serialized_end=143,
    methods=[
        _descriptor.MethodDescriptor(
            name="SendLint",
            full_name="lint.LintProto.SendLint",
            index=0,
            containing_service=None,
            input_type=_LINTREQUEST,
            output_type=_LINTREPLY,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
)
_sym_db.RegisterServiceDescriptor(_LINTPROTO)

DESCRIPTOR.services_by_name["LintProto"] = _LINTPROTO

# @@protoc_insertion_point(module_scope)
